/*
 * $Id: util.c,v 1.2 2001/09/06 12:07:44 joergland Exp $
 *
 * Copyright (C) 1999-2001 Joerg Wendland <joerg@wendlandnet.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <crypt.h>
#include "pam_pgsql.h"
#include "util.h"
#include "globalopts.h"

static const char *saltchars = 
"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

#define NSALTCHARS 62
#define MAXREAD 256

/* creates a pseudo-random salt with length 'saltlen' */
char 
*createsalt(unsigned int saltlen)
{
    struct timeval tv;
    unsigned int i;
    static char retval[MAX_SALTLEN+1];
    double r, m = RAND_MAX;

    if(saltlen > MAX_SALTLEN) saltlen = MAX_SALTLEN;

    /* use microsecond counter as seed for rand() */
    gettimeofday(&tv, NULL);
    srand(tv.tv_usec);
    
    for(i = 0; i < saltlen; i++) {
	r = rand();
	retval[i] = saltchars[abs(r/m*NSALTCHARS)];
    }
    retval[i] = '\0';

    return retval;
}

/* verifies a clear text against a crypted password */
BOOL 
vrfy_passwd(const char *clear, const char *crypted)
{
    char *tmp;

    D("vrfy_passwd");

    tmp = crypt(clear, crypted);

    if(mstrncmp(tmp, crypted) == 0)
	return TRUE;
    else
	return FALSE;

    /* should never be reached */
    return FALSE;
}

/* string copy with inline allocation and length check */
inline void 
mastrncpy(char **dst, char *src)
{
    int len;

    len = strlen(src);
    *dst = malloc(sizeof(char)*(len+1));
    memset(*dst, 0, len+1);
    strncpy(*dst, src, len);
}

inline int 
mstrncasecmp(const char *fst, const char *snd)
{
    unsigned int flen = strlen(fst), slen = strlen(snd);
    return strncasecmp(fst, snd, flen > slen ? slen : flen);
}

inline int 
mstrncmp(const char *fst, const char *snd)
{
    unsigned int flen = strlen(fst), slen = strlen(snd);
    return strncmp(fst, snd, flen > slen ? slen : flen);
}

/*
 * parse configfile
 * not perfect, but works
 */
void 
init_options()
{
#define NPARAMS 9
    struct _init_params {
	const char *name;
	char **val;
    } init_params[] = {
	{ "url",                  &__db_url                  },
	{ "user",                 &__db_user                 },
	{ "password",             &__db_password             },
	{ "auth_table",           &__db_auth_table           },
	{ "auth_login_field",     &__db_auth_login_field     },
	{ "auth_passwd_field",    &__db_auth_passwd_field    },
	{ "acct_table",           &__db_acct_table           },
	{ "acct_login_field",     &__db_acct_login_field     },
	{ "acct_enabled_field",   &__db_acct_enabled_field   },
	{ "acct_pwdexpire_field", &__db_acct_pwdexpire_field }
    };

    char line[MAXREAD], *tok;
    int i;
    FILE *cf = fopen("/etc/pam_pgsql.conf", "r");

    D(init_params[0].name);
    D("while");
    fgets(line, MAXREAD, cf);
    while(!feof(cf)) {
	tok = strtok(line, "=");
	D(tok);
	for(i = 0; i < NPARAMS; i++) {
	    if(!mstrncasecmp(init_params[i].name, tok)) {
		mastrncpy(init_params[i].val, strtok(NULL, "\n"));
		D(*(init_params[i].val));
		i = NPARAMS;
	    }
	}
	fgets(line, MAXREAD, cf);
    }

    fclose(cf);
}

void
_pam_log(int level, const char *format, ...)
{
    va_list args;
    va_start(args, format);
    openlog("pam_pgsql", LOG_PID, LOG_AUTH);
    vsyslog(level, format, args);
    closelog();
}

size_t sql_escape(const char *from, char *to, size_t len)
{
    const char *source = from;
    char *target = to;
    unsigned int remaining = len;

    while (remaining > 0) {
	switch (*source) {
	case '\0':
	    *target = '\\';
	    target++;
	    *target = '0';
	    break;

	case '\\':
	    *target = '\\';
	    target++;
	    *target = '\\';
	    break;

	case '\'':
	    *target = '\\';
	    target++;
	    *target = '\'';
	    break;

	case '"':
	    *target = '\\';
	    target++;
	    *target = '"';
	    break;

	default:
	    *target = *source;
	}
	source++;
	target++;
	remaining--;
    }

    *target = '\0';

    return target - to;
}
